#include <math.h>
#if defined(__APPLE__) && defined(__MACH__)
#include <OpenGL/gl.h>
#else
#include <GL/gl.h>
#endif
#include "cam.h"

#define DEG_TO_RAD(a) (((float)a) * (M_PI / 180.0))

static float cam_theta = 0, cam_phi = 25, cam_dist = 5;
static float cam_x, cam_y, cam_z;

static float cam_speed = 0.005;
static float rot_speed = 0.5;
static float zoom_speed = 0.1;

void cam_reset(void)
{
	cam_theta = 0;
	cam_phi = 25;
	cam_dist = 5;
	cam_x = cam_y = cam_z = 0.0;
}

void cam_pan(int dx, int dy)
{
	float dxf = dx * cam_speed * cam_dist;
	float dyf = dy * cam_speed * cam_dist;
	float angle = -DEG_TO_RAD(cam_theta);

	cam_x += cos(angle) * dxf + sin(angle) * dyf;
	cam_z += -sin(angle) * dxf + cos(angle) * dyf;
}

void cam_rotate(int dx, int dy)
{
	cam_theta += dx * rot_speed;
	cam_phi += dy * rot_speed;

	if(cam_phi <= -90) cam_phi = -89;
	if(cam_phi >= 90) cam_phi = 89;
}

void cam_zoom(int dz)
{
	cam_dist += dz * zoom_speed;
	if(cam_dist < 0.001) {
		cam_dist = 0.001;
	}
}

void set_view_matrix(void)
{
	glTranslatef(0, 0, -cam_dist);
	glRotatef(cam_phi, 1, 0, 0);
	glRotatef(cam_theta, 0, 1, 0);
	glTranslatef(-cam_x, -cam_y, -cam_z);
}
