#include <stdio.h>
#include <stdlib.h>
#if defined(__APPLE__) && defined(__MACH__)
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif
#include "text.h"

#define TEXT_PT_SIZE	18

static unsigned int fnt;
static int xsz, ysz;
static int max_descent;

int init_utkgl(void)
{
	int vp[4];

	if(!(fnt = create_font("georgia.ttf", TEXT_PT_SIZE))) {
		fprintf(stderr, "failed to load georgia.ttf\n");
		return -1;
	}

	max_descent = get_max_descent();

	glGetIntegerv(GL_VIEWPORT, vp);
	xsz = vp[2] - vp[0];
	ysz = vp[3] - vp[1];

	return 0;
}


#define CONVX(x)	(2.0 * (float)(x) / (float)xsz - 1.0)
#define CONVY(y)	(1.0 - 2.0 * (float)(y) / (float)ysz)

void utk_color(int r, int g, int b, int a)
{
	glColor4ub(r, g, b, a);
	set_text_color(r / 255.0, g / 255.0, b / 255.0, a / 255.0);
}

void utk_clip(int x1, int y1, int x2, int y2)
{
	if(x1 == x2 && y1 == y2 && x1 == y1 && x1 == 0) {
		glDisable(GL_SCISSOR_TEST);
	} else {
		glEnable(GL_SCISSOR_TEST);
	}
	glScissor(x1, ysz - y2, x2 - x1, y2 - y1);
}

void utk_image(int x, int y, const void *pix, int w, int h)
{
	glPixelZoom(1, -1);
	glRasterPos2f(CONVX(x), CONVY(y));
	glDrawPixels(w, h, GL_BGRA, GL_UNSIGNED_BYTE, pix);
}

void utk_rect(int x1, int y1, int x2, int y2)
{
	glRectf(CONVX(x1), CONVY(y1), CONVX(x2), CONVY(y2));
}

void utk_line(int x1, int y1, int x2, int y2, int width)
{
	glLineWidth((float)width);
	glBegin(GL_LINES);
	glVertex2f(CONVX(x1), CONVY(y1));
	glVertex2f(CONVX(x2), CONVY(y2));
	glEnd();
}

void utk_text(int x, int y, const char *txt, int sz)
{
	float fx = (float)x / (float)xsz;
	float fy = (float)y / (float)ysz;

	set_text_pos(fx, fy - max_descent);
	set_text_size((float)sz / (float)TEXT_PT_SIZE);
	print_string(txt);
}

int utk_text_spacing()
{
	return (int)(get_line_advance() * (float)ysz);
}

int utk_text_width(const char *txt, int sz)
{
	int width;
	float prev_sz = get_text_size();

	set_text_size((float)sz / (float)TEXT_PT_SIZE);

	width = (int)(get_text_width(txt) * (float)xsz);

	set_text_size(prev_sz);
	return width;
}

